@@ -95,12 +95,17 @@ xcalloc(size_t num, size_t size)
   return tmp;
 } /** end of xcalloc **/
 
+
+/** 
+ * free() wrapper:
+ * free it and NULL it to ensure we
+ * don't free it a second time...
+ */
 void
 xfree(void *ptr)
 {
-  if(ptr!=NULL){
-    free(ptr);
+  if(ptr!=(void *)NULL){
+    free(ptr); ptr=(void *)NULL;
   }
-  return;
-} 
+}
 
