@@ -30,26 +30,30 @@
  *                  start to start + length in a
  *                  larger char pointer (buffer).
  */
-char * 
-substring(char *buffer, int start, int length)
+char *
+substring(char *str, int start, int len)
 {
-  char *sub;
-	
-  if ((length < 1) || (start < 0) || (start > (int)strlen (buffer)))
+  int   i;
+  char  *ret;
+  char  *res;
+  char  *ptr;
+  char  *end;
+
+  if ((len < 1) || (start < 0) || (start > (int)strlen (str)) || start+len > (int)strlen(str))
     return NULL;
 
-  if ((int)strlen (buffer) < length){
-    sub = (char*) strdup (buffer);
-    return buffer;
-  }
+  ret = xmalloc(len+1);
+  res = ret;
+  ptr = str;
+  end = str;
 
-  sub = xmalloc(sizeof(char) * length + 1);
-  memset (sub, '\0', length + 1);
+  for(i = 0; i < start; i++, *ptr++) ;
+  for(i = 0; i < start+len; i++, *end++) ;
+  while(ptr < end)
+    *res++ = *ptr++;
 
-  buffer += start;
-  memcpy(sub, buffer, length);
-
-  return sub;
+  *res = 0;
+  return ret;
 }
 
 /**
