@@ -117,12 +117,15 @@ read_cfg_file(LINES *l, char *filename)
       hash_add(H, option, value); 
     }
     else{
-      while(strstr(line, "$")){
-        line = evaluate(H, line);
+    char *tmp = xstrdup(line);
+      while(strstr(tmp, "$")){
+        tmp = evaluate(H, tmp);
       }
       l->line = (char**)realloc(l->line, sizeof(char *) * (l->index + 1));
-      l->line[l->index] = (char *)strdup( line );
+      l->line[l->index] = (char *)strdup(tmp);
       l->index++;	
+      
+      free(tmp);
     }
     memset(line, 0, sizeof(line));
   }
@@ -154,7 +157,7 @@ read_cmd_line(LINES *l, char *url)
   if(strlen(head) == 0);
   else{
     l->line = (char**)realloc(l->line, sizeof(char *) * (l->index + 1));
-    l->line[l->index] = (char *)strdup( head );
+    l->line[l->index] = (char *)strdup(head);
     l->index++;
   }
   x++;
