@@ -133,7 +133,7 @@ private void
 
     (*(workptr->routine))(workptr->arg);
 
-    free(workptr);
+    xfree(workptr);
   }
  
   return(NULL);
@@ -281,14 +281,14 @@ crew_join(CREW crew, BOOLEAN finish, void **payload)
 void crew_destroy(CREW crew) {
   WORK  *workptr;
 
-  free(crew->threads);
+  xfree(crew->threads);
   while(crew->head != NULL){
     workptr  = crew->head; 
     crew->head = crew->head->next;
-    free(workptr);
+    xfree(workptr);
   }
  
-  free(crew);
+  xfree(crew);
 }
 
 /**
