@@ -1,7 +1,7 @@
 /**
  * Signal Handling
  *
- * Copyright (C) 2000-2007 by
+ * Copyright (C) 2000-2008 by
  * Jeffrey Fulmer - <jeff@joedog.org>, et al. 
  * This file is distributed as part of Siege 
  *
@@ -77,7 +77,9 @@ sig_handler(CREW crew)
   sigwait(&sigs, &gotsig);
   my.verbose = FALSE;
   fprintf(stderr, "\nLifting the server siege..."); 
-  crew_set_shutdown(crew, TRUE);
+//  crew_set_shutdown(crew, TRUE);
+  crew_cancel(crew);
+
 
 #if defined (hpux) || defined(__hpux)
 #else
@@ -93,7 +95,7 @@ sig_handler(CREW crew)
    */
   pthread_usleep_np(501125); 
 
-  crew_cancel(crew);
+//  crew_cancel(crew);
 
 #if defined(hpux) || defined(__hpux)
 #else
