@@ -32,6 +32,7 @@
 #define REQBUF  40960 
 
 private char *__parse_pair(char **str);
+private char *__dequote(char *str);
 
 /**
  * HTTPS tunnel; set up a secure tunnel with the
@@ -100,8 +101,8 @@ http_get(CONN *C, URL *U)
   int  rlen;
   char *protocol; 
   char *keepalive;
-  char authwww[128];
-  char authpxy[128];
+  char authwww[512];
+  char authpxy[512];
   char request[REQBUF];  
   char portstr[16];
   char fullpath[4096];
@@ -136,18 +137,25 @@ http_get(CONN *C, URL *U)
   keepalive = (C->connection.keepalive == TRUE)?"keep-alive":"close";
   get_cookie_header(pthread_self(), U->hostname, cookie); 
   if(C->auth.www){
-    rlen = snprintf(
-      authwww, sizeof(authwww), 
-      "Authorization: %s %s\015\012", 
-      (C->auth.type.www==BASIC)?"Basic":"Digest", my.auth.encode
-    );
+    if(C->auth.type.www==DIGEST){
+      char *tmp;
+      tmp = digest_generate_authorization(C->auth.wwwchlg, C->auth.wwwcred, "GET", fullpath);
+      rlen = snprintf(authwww, sizeof(authwww), "Authorization: %s\015\012", tmp);
+      free(tmp);
+    } else {
+      rlen = snprintf(authwww, sizeof(authwww), "Authorization: Basic %s\015\012", my.auth.encode);
+    }
   }
   if(C->auth.proxy){
-    rlen = snprintf(
-      authpxy, sizeof(authpxy), 
-      "Proxy-Authorization: %s %s\015\012", 
-      (C->auth.type.www==BASIC)?"Basic":"Digest", my.proxy.encode
-    );
+    if(C->auth.type.proxy==DIGEST){
+      char *tmp;
+
+      tmp = digest_generate_authorization(C->auth.proxychlg, C->auth.proxycred, "GET", fullpath);
+      rlen = snprintf( authpxy, sizeof(authpxy), "Proxy-Authorization: %s\015\012", tmp);
+      free(tmp);
+    } else  {
+      rlen = snprintf( authpxy, sizeof(authpxy), "Proxy-Authorization: Basic %s\015\012", my.proxy.encode);
+    }
   }
 
   /** 
@@ -233,20 +241,28 @@ http_post(CONN *C, URL *U)
   keepalive = (C->connection.keepalive == TRUE)?"keep-alive":"close";
   get_cookie_header(pthread_self(), U->hostname, cookie);
   if( C->auth.www ){
-    rlen = snprintf(
-      authwww, sizeof(authwww),
-      "Authorization: %s %s\015\012",
-      (C->auth.type.www==BASIC)?"Basic":"Digest", my.auth.encode
-    );
+    if(C->auth.type.www==DIGEST){
+      char *tmp;
+
+      tmp = digest_generate_authorization(C->auth.wwwchlg, C->auth.wwwcred, "GET", fullpath);
+      rlen = snprintf( authwww, sizeof(authwww), "Authorization: %s\015\012", tmp);
+      free(tmp);
+    } else {
+      rlen = snprintf( authwww, sizeof(authwww), "Authorization: Basic %s\015\012", my.auth.encode);
+    }
   }
   if( C->auth.proxy ){
-    rlen = snprintf(
-      authpxy, sizeof(authpxy),
-      "Proxy-Authorization: %s %s\015\012",
-      (C->auth.type.www==BASIC)?"Basic":"Digest", my.proxy.encode
-    );
-  } 
-    
+    if(C->auth.type.proxy==DIGEST){
+      char *tmp;
+
+      tmp = digest_generate_authorization(C->auth.proxychlg, C->auth.proxycred, "GET", fullpath);
+      rlen = snprintf( authpxy, sizeof(authpxy), "Proxy-Authorization: %s\015\012", tmp);
+      free(tmp);
+    } else  {
+      rlen = snprintf( authpxy, sizeof(authpxy), "Proxy-Authorization: Basic %s\015\012", my.proxy.encode);
+    }
+  }
+
   /* build a request string to
      pass to the server       */
   rlen = snprintf(
@@ -286,6 +302,15 @@ http_post(CONN *C, URL *U)
   return 0;
 }
 
+void
+http_free_headers(HEADERS *h)
+{
+  xfree(h->redirect);
+  xfree(h->auth.realm.proxy);
+  xfree(h->auth.realm.www);
+  xfree(h);
+}
+
 /**
  * returns HEADERS struct
  * reads from http/https socket and parses
@@ -399,42 +424,67 @@ http_read_headers(CONN *C, URL *U)
       }
     }
     if(strncasecmp(line, "www-authenticate: ", 18) == 0){
-      char *tmp;
-      h->auth.www = TRUE;
-      if(strncasecmp(line+19, "digest", 6) == 0){
-        h->auth.type.www = DIGEST;
+      char *tmp     = ""; 
+      char *option  = "", *value = "";
+      char *newline = (char*)line;
+      if(strncasecmp(line+18, "digest", 6) == 0){
+        newline += 24;
+        h->auth.type.www      = DIGEST;
+        h->auth.challenge.www = xstrdup(line+18);
       } else {
+        newline += 23;
         h->auth.type.www = BASIC;
       }
-      tmp = strchr(line, '=');
-      if(tmp==NULL){
-        debug("parse error: %s:%d", __FILE__, __LINE__);
-        return NULL;
-      }
-      tmp++;
-      if(tmp[0] == '"'){ 
-        tmp++; 
-        tmp[strlen(tmp)-1] = '\0'; 
-      }
-      strncpy(h->auth.realm.www, tmp, strlen(tmp)); 
-    }
+      while((tmp = __parse_pair(&newline)) != NULL){
+        option = tmp; 
+        while(*tmp && !ISSPACE((int)*tmp) && !ISSEPARATOR(*tmp))
+          tmp++;
+        *tmp++=0;
+        while(ISSPACE((int)*tmp) || ISSEPARATOR(*tmp))
+          tmp++; 
+        value  = tmp;
+        while(*tmp)
+          tmp++;
+        if(!strncasecmp(option, "realm", 5)){
+          if(value != NULL){
+	    h->auth.realm.www = xstrdup(__dequote(value));
+          } else {
+            h->auth.realm.www = xstrdup("");
+          }
+        }
+      } /* end of parse pairs */
+    } 
     if(strncasecmp(line, "proxy-authenticate: ", 20) == 0){
-      char *tmp;
-      h->auth.proxy = TRUE;
-      if(strncasecmp(line+21, "digest", 6) == 0){
-        h->auth.type.proxy = DIGEST;
+      char *tmp     = ""; 
+      char *option  = "", *value = "";
+      char *newline = (char*)line;
+      if(strncasecmp(line+20, "digest", 6) == 0){
+        newline += 26;
+        h->auth.type.proxy      = DIGEST;
+        h->auth.challenge.proxy = xstrdup(line+20);
       } else {
+        newline += 25;
         h->auth.type.proxy = BASIC;
-      }   
-      tmp = strchr(line, '=');
-      if(tmp==NULL){
-        debug("parse error: %s:%d", __FILE__, __LINE__);
-        return NULL;
       }
-      tmp++;
-      if(tmp[0] == '"'){ tmp++; tmp[strlen(tmp)-1] = '\0'; }
-      strncpy(h->auth.realm.proxy, tmp, strlen(tmp)); 
-    }                      
+      while((tmp = __parse_pair(&newline)) != NULL){
+        option = tmp; 
+        while(*tmp && !ISSPACE((int)*tmp) && !ISSEPARATOR(*tmp))
+          tmp++;
+        *tmp++=0;
+        while(ISSPACE((int)*tmp) || ISSEPARATOR(*tmp))
+          tmp++; 
+        value  = tmp;
+        while(*tmp)
+          tmp++;
+        if(!strncasecmp(option, "realm", 5)){
+          if(value != NULL){
+	    h->auth.realm.proxy = xstrdup(__dequote(value));
+          } else {
+            h->auth.realm.proxy = xstrdup("");
+          }
+        }
+      } /* end of parse pairs */
+    }
     if(strncasecmp(line, "transfer-encoding: ", 19) == 0){
       if(strncasecmp(line+20, "chunked", 7)){
         C->content.transfer = CHUNKED; 
@@ -452,6 +502,7 @@ http_read_headers(CONN *C, URL *U)
     }
     if(n <=  0){ 
       debug("read error: %s:%d", __FILE__, __LINE__);
+      http_free_headers(h);
       return(NULL); 
     } /* socket closed */
   } /* end of while TRUE */
@@ -587,3 +638,40 @@ __parse_pair(char **str)
   else
     return NULL;
 } 
+
+char *
+__rquote(char *str)
+{
+  char *ptr;
+  int   len;
+
+  len = strlen(str);
+  for(ptr = str + len - 1; ptr >= str && ISQUOTE((int)*ptr ); --ptr);
+
+  ptr[1] = '\0';
+
+  return str;
+}
+
+char *
+__lquote(char *str)
+{
+  char *ptr;
+  int  len;
+
+  for(ptr = str; *ptr && ISQUOTE((int)*ptr); ++ptr);
+
+  len = strlen(ptr);
+  memmove(str, ptr, len + 1);
+
+  return str;
+}
+
+char *
+__dequote(char *str)
+{
+  char *ptr;
+  ptr = __rquote(str);
+  str = __lquote(ptr);
+  return str;
+}
