@@ -46,29 +46,34 @@
 typedef 
 struct headers
 {
-  char                head[64];                /* http response       */
-  int                 code;                    /* http return code    */
+  char                head[64];                /* http response         */
+  int                 code;                    /* http return code      */
   unsigned long int   length;
-  char                cookie[MAX_COOKIE_SIZE]; /* set-cookie data     */
-  char                *redirect;               /* redirection URL     */
+  char                cookie[MAX_COOKIE_SIZE]; /* set-cookie data       */
+  char                *redirect;               /* redirection URL       */
   struct{
-    int   www;                                 /* www auth challenge  */
-    int   proxy;                               /* proxy auth challenge*/
-    struct{
-      char  www[64];                           /* www auth realm      */
-      char  proxy[64];                         /* proxy auth realm    */
+    int   www;                                 /* www auth challenge    */
+    int   proxy;                               /* proxy auth challenge  */
+    struct {
+      char  *www;                              /* www auth realm        */
+      char  *proxy;                            /* proxy auth realm      */
     } realm;
-    struct{
+    struct {
+      char  *www;                              /* www auth realm        */
+      char  *proxy;                            /* proxy auth realm      */
+    } challenge;                               /* square peg/round hole */
+    struct {
       TYPE  www;
       TYPE  proxy;
     } type;
   } auth;
-  int                 keepalive;               /* http keep-alive     */
+  int                 keepalive;               /* http keep-alive       */
 } HEADERS;
 
 /* http function prototypes */
 int      http_get (CONN *C, URL *U);
 int      http_post(CONN *C, URL *U);
+void     http_free_headers(HEADERS *h);
 HEADERS *http_read_headers(CONN *C, URL *U);
 ssize_t  http_read(CONN *C);
 BOOLEAN  https_tunnel_request(CONN *C, char *host, int port);
