@@ -31,12 +31,17 @@
 #include <joedog/boolean.h>
 #include <joedog/defs.h>
 #include <joedog/joedog.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <errno.h>
 
 int
 init_config( void )
 {
   char *e;
   int  res;
+  struct stat buf;
 
   /* Check if we were passed the -R switch to use a different siegerc file.
    * If not, check for the presence of the SIEGERC variable, otherwise
@@ -46,6 +51,9 @@ init_config( void )
       snprintf(my.rc, sizeof(my.rc), e);
     } else {
       snprintf(my.rc, sizeof(my.rc), "%s/.siegerc", getenv("HOME"));
+      if (stat(my.rc, &buf) < 0 && errno == ENOENT) {
+        snprintf(my.rc, sizeof(my.rc), CNF_FILE);
+      }
     }
   } 
 
@@ -61,8 +69,8 @@ init_config( void )
   my.login          = FALSE;
   my.failures       = 1024;
   my.failed         = 0;
-  my.auth.encode    = (char*)"";
-  my.proxy.encode   = (char*)"";
+  my.auth.encode    = xstrdup("");
+  my.proxy.encode   = xstrdup("");
   my.proxy.required = FALSE;
   my.proxy.port     = 80; 
   my.timeout        = 30;
@@ -156,7 +164,7 @@ show_config( int EXIT )
   printf( "benchmark mode:                 %s\n", my.bench?"true":"false"  );
   printf( "failures until abort:           %d\n", my.failures );
   printf( "named URL:                      %s\n", my.url==NULL||strlen(my.url)<2?"none":my.url );
-  printf( "URLs file:                      %s\n", strlen(my.file)>1?my.file:CNF_FILE );
+  printf( "URLs file:                      %s\n", strlen(my.file)>1?my.file:URL_FILE );
   printf( "logging:                        %s\n", my.logging?"true":"false" );
   printf( "log file:                       %s\n", my.logfile==NULL?LOG_FILE:my.logfile );
   printf( "resource file:                  %s\n", my.rc);
@@ -222,6 +230,7 @@ load_conf(char *filename)
   char *line;
   char *tmp;
   char *option;
+  char *optionptr;
   char *value;
  
   if ((fp = fopen(filename, "r")) == NULL) {
@@ -232,13 +241,13 @@ load_conf(char *filename)
 
   while((line = chomp_line(fp, &line, &line_num)) != NULL){
     tmp    = line;
-    option = line;
-    while(*line && !ISSPACE((int)*line) && !ISSEPARATOR(*line))
-      line++;
-    *line++=0;
-    while(ISSPACE((int)*line) || ISSEPARATOR(*line))
-      line++;
-    value  = line;
+    optionptr = option = xstrdup(line);
+    while(*optionptr && !ISSPACE((int)*optionptr) && !ISSEPARATOR(*optionptr))
+      optionptr++;
+    *optionptr++=0;
+    while(ISSPACE((int)*optionptr) || ISSEPARATOR(*optionptr))
+      optionptr++;
+    value  = xstrdup(optionptr);
     while(*line)
       line++;  
     while(strstr(option, "$")){
@@ -367,14 +376,19 @@ load_conf(char *filename)
         my.loginurl = stralloc(value);
       } else {
         /* user login info */
-        char *usr, *pwd, *rlm;
-        usr = value;
-        while( *value && *value != ':' && *value != '\0' )
-          value++;
-        *value++=0; pwd = value;
-        while( *value && *value != ':' && *value != '\0' )
-        value++;
-        *value++=0; rlm = value;
+        char *usr, *pwd, *rlm, *tmpvalue;
+        usr = tmpvalue = value;
+        while( *tmpvalue && *tmpvalue != ':' && *tmpvalue != '\0' )
+          tmpvalue++;
+        *tmpvalue++=0; pwd = tmpvalue;
+        while( *tmpvalue && *tmpvalue != ':' && *tmpvalue != '\0' )
+	  tmpvalue++;
+	if('\0' != *tmpvalue) {
+	  *tmpvalue++=0;
+	  rlm = tmpvalue;
+	} else {
+	  rlm = NULL;
+	}
         add_authorization(WWW, usr, pwd, rlm);
       }
     }
@@ -414,14 +428,19 @@ load_conf(char *filename)
       }
     } 
     else if(strmatch(option, "proxy-login")){
-      char *usr, *pwd, *rlm;
-      usr = value;
-      while(*value && *value != ':' && *value != '\0')
-        value++;
-      *value++=0; pwd = value;
-      while(*value && *value != ':' && *value != '\0')
-      value++;
-      *value++=0; rlm = value;
+      char *usr, *pwd, *rlm, *tmpvalue;
+      usr = tmpvalue = value;
+      while(*tmpvalue && *tmpvalue != ':' && *tmpvalue != '\0')
+        tmpvalue++;
+      *tmpvalue++=0; pwd = tmpvalue;
+      while(*tmpvalue && *tmpvalue != ':' && *tmpvalue != '\0')
+      tmpvalue++;
+      if('\0' != *tmpvalue) {
+	*tmpvalue++=0;
+	rlm = tmpvalue;
+      } else {
+        rlm = NULL;
+      }
       add_authorization(PROXY, usr, pwd, rlm);  
     }
     else if(strmatch(option, "failures")){
@@ -485,6 +504,8 @@ load_conf(char *filename)
     } else {
       hash_add(H, option, value);
     }
+    xfree(value);
+    xfree(option);
     free(tmp);
   } /* end of while line=chomp_line */
 
