@@ -399,7 +399,10 @@ load_file(URL *U, char *file)
   memset(postdata, 0, POST_BUF);
 
   if((lstat(filename, &st) == 0) || (errno != ENOENT)){ 
-    len = st.st_size;  
+    len = (st.st_size >= POST_BUF) ? POST_BUF : st.st_size;  
+    if(len < (unsigned)st.st_size){
+      joe_warning("Truncated file: %s exceeds the post limit of %d bytes.\n", filename, POST_BUF);
+    }
     if((fp = fopen(filename, "r")) == NULL){
       joe_error("could not open file: %s", filename);
       return;
