@@ -230,7 +230,11 @@ parse_cmdline(int argc, char *argv[])
         my.verbose = TRUE;
         break;
       case 'r':
-        my.reps = atoi(optarg);
+        if(strmatch(optarg, "once")){
+           my.reps = -1;
+        } else {
+          my.reps = atoi(optarg);
+        }
         break;
       case 't':
         parse_time(optarg);
@@ -299,7 +303,7 @@ main(int argc, char *argv[])
   sigaddset(&sigs, SIGTERM);
   sigprocmask(SIG_BLOCK, &sigs, NULL);
 
-  lines = xcalloc(sizeof(LINES),1);
+  lines = xcalloc(1, sizeof *lines);
   lines->index   = 0;
   lines->line    = NULL;
 
@@ -326,7 +330,11 @@ main(int argc, char *argv[])
   if(my.url != NULL){
     my.length = 1; 
   } else { 
-    my.length = read_cfg_file( lines, my.file ); 
+    my.length = read_cfg_file(lines, my.file); 
+  }
+
+  if(my.reps < 0){
+    my.reps = my.length;
   }
 
   if(my.length == 0){ 
@@ -342,7 +350,7 @@ main(int argc, char *argv[])
 
   /* memory allocation for threads and clients */
   urls   = xmalloc(my.length * sizeof(URL));
-  client = xmalloc(sizeof(CLIENT) * my.cusers);
+  client = xcalloc(my.cusers, sizeof(CLIENT));
   if((crew = new_crew(my.cusers, my.cusers, FALSE)) == NULL){
     joe_fatal("unable to allocate memory for %d simulated browser", my.cusers);  
   }
@@ -476,6 +484,21 @@ main(int argc, char *argv[])
    * cleanup crew
    */ 
   crew_destroy(crew);
+  for(x = 0; x < my.length; x++) {
+    if(urls[x] != NULL){
+       xfree(urls[x]->pathname);
+       xfree(urls[x]->hostname);
+       xfree(urls[x]);
+     }
+   }
+  for(x = 0; x < my.cusers; x++){
+    digest_challenge_destroy(client[x].auth.wwwchlg);
+    digest_credential_destroy(client[x].auth.wwwcred);
+    digest_challenge_destroy(client[x].auth.proxychlg);
+    digest_credential_destroy(client[x].auth.proxycred);
+  }
+  xfree(client);
+  xfree(urls);
 
   if(my.get){
     if(data_get_count(D) > 0){
@@ -532,10 +555,40 @@ main(int argc, char *argv[])
        xfree(lines->line[x]);
     xfree(lines->line);
     xfree(lines);
-  }  
+  } else {
+    xfree(lines->line);
+    xfree(lines);
+  }
+  if(my.auth.encode != NULL)
+    xfree(my.auth.encode);
+  if(my.proxy.hostname != NULL)
+    xfree(my.proxy.hostname);
+  if(my.proxy.encode != NULL)
+    xfree(my.proxy.encode);
 
-  exit(EXIT_SUCCESS);	
+  while(my.auth.head != NULL) {
+    struct LOGIN *current;
+    current = my.auth.head;
+    my.auth.head = current->next;
+    xfree(current->username);
+    xfree(current->password);
+    xfree(current->realm);
+    xfree(current);
+  }
+  while(my.proxy.head != NULL) {
+    struct LOGIN *current;
+    current = my.proxy.head;
+    my.proxy.head = current->next;
+    xfree(current->username);
+    xfree(current->password);
+    xfree(current->realm);
+    xfree(current);
+  }
+
+  pthread_mutex_destroy( &(cookie->mutex));
+  /* should probably take a deeper look at cookie content to free it */
+  xfree(cookie);
+  xfree(my.url);
+
+  exit(EXIT_SUCCESS);  
 } /* end of int main **/
-
-
-
