@@ -78,7 +78,7 @@ new_socket(CONN *C, const char *hostparam, int portparam)
   int herrno;
   struct sockaddr_in cli; 
   struct hostent     *hp;
-  char   hn[256];
+  char   hn[512];
   int    port;
 #if defined(_AIX)
   char *aixbuf;
@@ -86,6 +86,8 @@ new_socket(CONN *C, const char *hostparam, int portparam)
 #endif/*_AIX*/ 
 
   C->encrypt = FALSE;
+
+  memset(hn, '\0', sizeof hn);
  
   /* if we are using a proxy, then we make a socket
      connection to that server rather then a httpd */ 
@@ -99,7 +101,7 @@ new_socket(CONN *C, const char *hostparam, int portparam)
 
   /* create a socket, return -1 on failure */
   if((C->sock = socket(AF_INET, SOCK_STREAM, 0)) < 0){
-    switch( errno ){
+    switch(errno){
       case EPROTONOSUPPORT: { joe_error("unsupported protocol %s:%d",  __FILE__, __LINE__); break; }
       case EMFILE:          { joe_error("descriptor table full %s:%d", __FILE__, __LINE__); break; }
       case ENFILE:          { joe_error("file table full %s:%d",       __FILE__, __LINE__); break; }
@@ -114,10 +116,11 @@ new_socket(CONN *C, const char *hostparam, int portparam)
 
 #if defined(__GLIBC__)
   {
-    struct hostent hent; 
-    char hbf[9000]; 
+    struct hostent hent;
+    char hbf[8192]; 
+    memset(hbf, '\0', sizeof hbf);
     /* for systems using GNU libc */
-    if((gethostbyname_r( hn, &hent, hbf, sizeof(hbf), &hp, &herrno ) < 0)){
+    if((gethostbyname_r(hostparam, &hent, hbf, sizeof(hbf), &hp, &herrno) < 0)){
       hp = NULL;
     }
   }
@@ -126,13 +129,14 @@ new_socket(CONN *C, const char *hostparam, int portparam)
   hp = getipnodebyname(hn, AF_INET, 0, &herrno);
 # else /* default use gethostbyname_r*/
   {
-    struct hostent hent; 
-    char hbf[9000]; 
+    struct hostent hent;
+    char hbf[8192]; 
+    memset(hbf, '\0', sizeof hbf);
     hp = gethostbyname_r(hn, &hent, hbf, sizeof(hbf), &herrno); 
   }
 # endif/*HAVE_GETIPNODEBYNAME*/
 #elif defined(_AIX)
-  aixbuf = (char*)xmalloc(9000);
+  aixbuf = (char*)xmalloc(8192);
   rc  = gethostbyname_r(hn, (struct hostent *)aixbuf,
                        (struct hostent_data *)(aixbuf + sizeof(struct hostent)));
   hp = (struct hostent*)aixbuf;
@@ -199,7 +203,7 @@ new_socket(CONN *C, const char *hostparam, int portparam)
     FD_ZERO(&ws);
     FD_SET(C->sock, &rs);
     FD_SET(C->sock, &ws);
-    memset((void *)&timeout, '\0', sizeof( struct timeval ));
+    memset((void *)&timeout, '\0', sizeof(struct timeval));
     timeout.tv_sec  = (my.timeout > 0)?my.timeout:30;
     timeout.tv_usec = 0;
     res = select(C->sock+1, &rs, &ws, NULL, &timeout);
@@ -419,15 +423,15 @@ socket_read(CONN *C, void *vbuf, size_t len)
   int ret_eof = 0;
  
   pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &type);
-  if(__socket_check(C, READ) == FALSE){
-    return -1;
-  }
  
   buf = vbuf;
   n   = len;
   if(C->encrypt == TRUE){
   #ifdef HAVE_SSL
     while(n > 0){
+      if(__socket_check(C, READ) == FALSE){
+	return -1;
+      }
       if((r = SSL_read(C->ssl, buf, n)) < 0){
         if( errno == EINTR )
           r = 0;
@@ -441,10 +445,18 @@ socket_read(CONN *C, void *vbuf, size_t len)
   #endif/*HAVE_SSL*/
   } else { 
     while(n > 0){
+      if(C->inbuffer < len){
+        if(__socket_check(C, READ) == FALSE){
+          return -1;
+        }
+      }
       if(C->inbuffer <  n){
         int lidos;
         memmove(C->buffer,&C->buffer[C->pos_ini],C->inbuffer);
         C->pos_ini = 0;
+	if(__socket_check(C, READ) == FALSE){
+	  return -1;
+	}
         lidos = read(C->sock, &C->buffer[C->inbuffer], sizeof(C->buffer)-C->inbuffer);
         if(lidos == 0)
           ret_eof = 1;
