@@ -95,8 +95,12 @@ typedef struct
     int  keepalive;    /* keep-alive header directive     */
     int  tested;       /* boolean, has socket been tested */
   } connection;        /* persistent connection data      */
-  struct{
+  struct {
+    DIGEST_CHLG *wwwchlg;
+    DIGEST_CRED *wwwcred;
     int    www;
+    DIGEST_CHLG *proxychlg;
+    DIGEST_CRED *proxycred;
     int    proxy;
     struct{
       TYPE www;
