@@ -15,6 +15,20 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -57,8 +71,11 @@ SSL_initialize(CONN *C)
   C->method = NULL;
   C->cert   = NULL; 
   
-  SSL_load_error_strings();
-  SSL_library_init();
+  /** 
+   * XXX: SSL_library_init(); 
+   *      SSL_load_error_strings();
+   * moved to ssl.c:235 - use once 
+   */
   if(!my.ssl_key && my.ssl_cert) {
     my.ssl_key = my.ssl_cert;
   }
@@ -142,7 +159,9 @@ SSL_thread_setup( void )
    );
    pthread_sleep_np( 2 );
 #endif
- 
+
+  SSL_library_init();
+  SSL_load_error_strings();
   lock_cs    = (pthread_mutex_t*)OPENSSL_malloc(
     CRYPTO_num_locks()*sizeof(pthread_mutex_t)
   );
@@ -159,16 +178,22 @@ SSL_thread_setup( void )
 }
 
 void 
-SSL_thread_cleanup( void ) 
+SSL_thread_cleanup(void) 
 {
   int x;
   
   CRYPTO_set_locking_callback(NULL);
-  for( x = 0; x < CRYPTO_num_locks(); x++ ) {
+  for(x = 0; x < CRYPTO_num_locks(); x++){
     pthread_mutex_destroy(&(lock_cs[x]));
   }
-  OPENSSL_free(lock_cs);
-  OPENSSL_free(lock_count);
+  if(lock_cs!=(pthread_mutex_t *)NULL) { 
+    OPENSSL_free(lock_cs); 
+    lock_cs=(pthread_mutex_t *)NULL; 
+  }
+  if(lock_count!=(long *)NULL){ 
+    OPENSSL_free(lock_count); 
+    lock_count=(long *)NULL; 
+  }
 }
 
 void 
