@@ -412,7 +412,7 @@ build_url(char *url, int defaultport, int id)
     xfree(portstr); 
   }
 
-  tmp = substring(url, mark[2], strlen(url));
+  tmp = substring(url, mark[2], strlen(url) - mark[2]);
   if(tmp == NULL){
     U->pathname = (char *)xstrdup("/"); 
   } else {
@@ -427,25 +427,10 @@ build_url(char *url, int defaultport, int id)
     (U->protocol==HTTP)?"http://":"https://", U->hostname, U->port, U->pathname
   );
 
-
-  xfree(url); 
   xfree(tmp);
   return(U);
 }
 
-URL *
-add_redirect(char *url, int port, int id)
-{
-  char *tmp;
-
-  /**
-   * freed in build_url()
-   */
-  tmp = xstrdup(url);
- 
-  return build_url(tmp, port, id);
-}
-
 /**
  * add_url
  * parses char * then populates and 
@@ -454,6 +439,7 @@ add_redirect(char *url, int port, int id)
 URL *
 add_url(char *url, int id)
 {
+  URL *tmp_url;
   char *tmp;
 
   /**
@@ -469,7 +455,10 @@ add_url(char *url, int id)
    * freed in build_url()
    */
   tmp = (char*)xstrdup(url);
-  return build_url(tmp, -1, id);
+  tmp_url = build_url(tmp, -1, id);
+  xfree(tmp);
+  
+  return tmp_url;
 }
 
 
